
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <linux/kd.h>

/*
 * LED routines
 */

static int cons_fd = -1;
static char saved_led_settings;

static void led_close ()
{
   ioctl (cons_fd,KDSETLED,saved_led_settings);
   close (cons_fd);
}

static void led_open ()
{
   if ((cons_fd = open ("/dev/console",O_RDWR)) < 0)
	 {
		perror ("open(/dev/console,O_RDWR)");
		exit (EXIT_FAILURE);
	 }
   if (ioctl (cons_fd,KDGETLED,&saved_led_settings) < 0)
	 {
		perror ("ioctl(KDGETLED)");
		close (cons_fd);
		exit (EXIT_FAILURE);
	 }
   atexit (led_close);
}

static void led_write (char ledval)
{
   if (ioctl (cons_fd,KDSETLED,ledval) < 0)
	 {
		perror ("ioctl(KDSETLED)");
		exit (EXIT_FAILURE);
	 }
}

/*
 * Key routines
 */

static struct termios saved_terminal_settings;

static void key_close ()
{
   tcsetattr (STDIN_FILENO,TCSANOW,&saved_terminal_settings);
}

static void key_open ()
{
   struct termios tval;
   if (tcgetattr (STDIN_FILENO,&saved_terminal_settings) < 0 || tcgetattr (STDIN_FILENO,&tval) < 0)
	 {
		perror ("tcgetattr(STDIN_FILENO)");
		exit (EXIT_FAILURE);
	 }

   /* disable character echoing */
   tval.c_lflag &= ~ECHO;

   /* disable canonical mode and set buffer size to 1 byte */
   tval.c_lflag &= ~ICANON;
   tval.c_cc[VTIME] = 0;
   tval.c_cc[VMIN] = 1;

   if (tcsetattr (STDIN_FILENO,TCSANOW,&tval) < 0)
	 {
		perror ("tcsetattr(STDIN_FILENO)");
		exit (EXIT_FAILURE);
	 }
   atexit (key_close);
}

static int key_read ()
{
   char c;
   struct timeval tv;
   fd_set rfds;
   FD_ZERO (&rfds);
   FD_SET (STDIN_FILENO,&rfds);
   tv.tv_usec = 0;
   tv.tv_sec = 1;	/* wait 1 second for a keypress */
   if (select (STDIN_FILENO + 1,&rfds,NULL,NULL,&tv) < 0)
	 {
		perror ("select(STDIN_FILENO)");
		exit (EXIT_FAILURE);
	 }
   switch (read (STDIN_FILENO,&c,1))
	 {
	  case 0:
		fprintf (stderr,"No characters available to read!\n");
		exit (EXIT_FAILURE);
	  case 1:
		break;
	  default:
		perror ("read(STDIN_FILENO)");
	 }
   return (c);
}

int main ()
{
   int i;

   led_open ();
   key_open ();

   /* talk to ipac */
   led_write (0);
   for (i = 0; i < 5; i++) led_write (LED_NUM | LED_CAP | LED_SCR);

   /* what does it say? */
   switch ((i = key_read ()))
	 {
	  case 'Y':
		printf ("Ready to program\n");
		break;
	  case 'J':
		printf ("Jumper set to fixed\n");
		break;
	  case -1:
		exit (EXIT_FAILURE);
	  default:
		fprintf (stderr,"Unknown error code received: %c\n",i);
		exit (EXIT_FAILURE);
	 }

   led_close ();
   exit (EXIT_SUCCESS);
}

